import * as Platform from '@nativescript/core/platform';
import { AbsoluteLayout, GridLayout, GridUnitType, ItemSpec, ScrollView, StackLayout, Application as app } from '@nativescript/core';
import { ParallaxUtilities } from './utilities';
export class Header extends StackLayout {
    constructor() {
        super();
    }
}
export class Anchored extends StackLayout {
    constructor() {
        super();
        this.dropShadow = false;
    }
    get dropShadow() {
        return this._dropShadow;
    }
    set dropShadow(value) {
        this._dropShadow = value;
    }
}
export class Content extends StackLayout {
    constructor() {
        super();
    }
}
export class ParallaxView extends GridLayout {
    constructor() {
        super();
        this._childLayouts = [];
        let headerView;
        let contentView;
        let scrollView = new ScrollView();
        let viewsToFade;
        let maxTopViewHeight;
        let controlsToFade;
        let anchoredRow = new AbsoluteLayout();
        let row = new ItemSpec(2, GridUnitType.STAR);
        let column = new ItemSpec(1, GridUnitType.STAR);
        let invalidSetup = false;
        let hasBeenAnchored = false;
        this._isAnchored = false;
        this.bounce = false;
        this._minimumHeights = ParallaxUtilities.getMinimumHeights();
        this.verticalAlignment = 'top';
        scrollView.verticalAlignment = 'top';
        anchoredRow.verticalAlignment = 'top';
        this._includesAnchored = false;
        this._topOpacity = 1;
        this._loaded = false;
        this.on(GridLayout.loadedEvent, (data) => {
            if (!this._loaded) {
                this._loaded = true;
                this._childLayouts = ParallaxUtilities.pluckViews(this);
                this.addRow(row);
                this.addColumn(column);
                this.addChild(scrollView);
                this.addChild(anchoredRow);
                GridLayout.setRow(scrollView, 1);
                GridLayout.setRow(anchoredRow, 0);
                GridLayout.setColumn(scrollView, 1);
                GridLayout.setColumn(anchoredRow, 0);
                let wrapperStackLayout = new StackLayout();
                scrollView.content = wrapperStackLayout;
                for (var c = 0; c < this._childLayouts.length; c++) {
                    var element = this._childLayouts[c];
                    if (element instanceof Header || (StackLayout && ParallaxUtilities.containsCssClass(element, 'header'))) {
                        wrapperStackLayout.addChild(element);
                        headerView = element;
                    }
                    else if (element instanceof Content || (StackLayout && ParallaxUtilities.containsCssClass(element, 'content'))) {
                        wrapperStackLayout.addChild(element);
                        contentView = element;
                    }
                    else if (element instanceof Anchored || (StackLayout && ParallaxUtilities.containsCssClass(element, 'anchor'))) {
                        anchoredRow.addChild(element);
                        if (element.dropShadow || ParallaxUtilities.containsCssClass(element, 'dropShadow')) {
                            anchoredRow.height = element.height;
                            anchoredRow.addChild(ParallaxUtilities.addDropShadow(element.height, element.getMeasuredWidth()));
                        }
                        else {
                            anchoredRow.height = element.height;
                        }
                        element.verticalAlignment = 'top';
                        this._includesAnchored = true;
                    }
                }
                ;
                if (headerView == null || contentView == null) {
                    ParallaxUtilities.displayDevWarning(this, 'Parallax ScrollView Setup Invalid. You must have Header and Content tags', headerView, contentView, contentView);
                    return;
                }
                if (isNaN(headerView.height)) {
                    ParallaxUtilities.displayDevWarning(this, 'Header MUST have a height set.', headerView, anchoredRow, contentView);
                    return;
                }
                if (this._includesAnchored && isNaN(anchoredRow.height)) {
                    ParallaxUtilities.displayDevWarning(this, 'Anchor MUST have a height set.', anchoredRow, headerView, contentView);
                    return;
                }
                maxTopViewHeight = headerView.height;
                if (this._includesAnchored) {
                    anchoredRow.translateY = maxTopViewHeight;
                    if (app.android) {
                        anchoredRow.translateY = anchoredRow.translateY - 5;
                    }
                    contentView.translateY = anchoredRow.height;
                }
                if (this._bounce == "false" || this._bounce === false) {
                    if (app.ios) {
                        scrollView.ios.bounces = false;
                    }
                    else if (app.android) {
                        scrollView.android.setOverScrollMode(2);
                    }
                }
                viewsToFade = [];
                if (this.controlsToFade == null) {
                    controlsToFade = [];
                }
                else {
                    controlsToFade = this.controlsToFade.split(',');
                }
                for (var c = 0; c < controlsToFade.length; c++) {
                    let newView = headerView.getViewById(controlsToFade[c]);
                    if (newView != null) {
                        viewsToFade.push(newView);
                    }
                }
                ;
                let prevOffset = -10;
                ParallaxUtilities.setMinimumHeight(contentView, anchoredRow, Platform.Screen.mainScreen.heightDIPs, this._includesAnchored);
                app.on(app.orientationChangedEvent, (args) => {
                    ParallaxUtilities.setMinimumHeight(contentView, anchoredRow, this._minimumHeights[args.newValue], this._includesAnchored);
                });
                scrollView.on(ScrollView.scrollEvent, (args) => {
                    this.notify({
                        eventName: ParallaxView.scrollEvent,
                        object: this,
                        data: args,
                        direction: (prevOffset > args.scrollY) ? "down" : "up"
                    });
                    if (this._includesAnchored) {
                        anchoredRow.translateY = ParallaxUtilities.getAnchoredTopHeight(maxTopViewHeight, scrollView.verticalOffset);
                        if (anchoredRow.translateY === 0) {
                            if (!this._isAnchored) {
                                this._isAnchored = true;
                                hasBeenAnchored = true;
                                this.notify({
                                    eventName: ParallaxView.anchoredEvent,
                                    object: this,
                                    data: anchoredRow
                                });
                            }
                        }
                        else if (anchoredRow.translateY > 0) {
                            if (hasBeenAnchored && this._isAnchored) {
                                this.notify({
                                    eventName: ParallaxView.unanchoredEvent,
                                    object: this,
                                    data: anchoredRow
                                });
                            }
                            this._isAnchored = false;
                        }
                    }
                    headerView.height = ParallaxUtilities.getTopViewHeight(maxTopViewHeight, scrollView.verticalOffset);
                    if (headerView.height > 0) {
                        ParallaxUtilities.fadeViews(maxTopViewHeight, scrollView.verticalOffset, viewsToFade, this._topOpacity);
                    }
                    prevOffset = scrollView.verticalOffset;
                });
            }
        });
    }
    get bounce() {
        return this._bounce;
    }
    set bounce(value) {
        this._bounce = value;
    }
    get controlsToFade() {
        return this._controlsToFade;
    }
    set controlsToFade(value) {
        this._controlsToFade = value;
    }
    get isAnchored() {
        return this._isAnchored;
    }
}
ParallaxView.scrollEvent = "scroll";
ParallaxView.anchoredEvent = "anchored";
ParallaxView.unanchoredEvent = "unanchored";
//# sourceMappingURL=nativescript-parallax.js.map